﻿using UnityEngine;
using System.Collections;
using System;
using System.Linq;
using System.Runtime.InteropServices;


// you only need this class defined once in any given unity project
// these are the RTcmix functions needed to run and control RTcmix
public static class DLL
{
	[DllImport ("librtcmix_embedded")]
	public static extern int RTcmix_init(int objno);

	[DllImport ("librtcmix_embedded")]
	public static extern int RTcmix_setparams(float sr, int nchans, int vecsize, int recording, int bus_count, int objno);

	[DllImport ("librtcmix_embedded")]
	public static extern int RTcmix_setAudioBufferFormat(int format, int nchans, int objno);

	[DllImport("librtcmix_embedded")]
	unsafe public static extern int RTcmix_runAudio(void *k, void *outAudioBuffer, int nframes, int objno);

	[DllImport ("librtcmix_embedded")]
	unsafe public static extern int RTcmix_destroy(int objno);

	[DllImport ("librtcmix_embedded")]
	unsafe public static extern int unity_parse_score(char *buf, int len, int objno);

	[DllImport ("librtcmix_embedded")]
	unsafe public static extern int  unity_checkForBang(int objno);

	[DllImport ("librtcmix_embedded")]
	unsafe public static extern int  unity_checkForVals(float *thevals, int objno);

	[DllImport ("librtcmix_embedded")]
	unsafe public static extern int unity_checkForPrint(char *pptr, int objno);

	[DllImport ("librtcmix_embedded")]
	unsafe public static extern void RTcmix_setPField(int inlet, float pval, int objno);

	[DllImport ("librtcmix_embedded")]
	unsafe public static extern int check_context();
}


// change “rtcmixtemplate” to the name of your .cs script
public class rtcmixmain : MonoBehaviour {
	int objno = 0; // change this for multiple RTcmix instantiations
	float[] vals = new float[1024]; // for MAXMESSAGE values
	char[] printvals = new char[8192]; // for printing RTcmix messages/errors

	void Start () {
		int status;

		while (DLL.check_context() == 1) System.Threading.Thread.Sleep (1);
		status = DLL.RTcmix_init(objno);

		// set it for single-precision float samples (see RTcmix_API.h), scaled between -1.0 and 1.0, 2 chans;
		while (DLL.check_context() == 1) System.Threading.Thread.Sleep (1);
		DLL.RTcmix_setAudioBufferFormat(8, 2, objno);

		// note the "1024" here, vs. "2048" in the Unity code
		// this is the difference between 'frames' and 'nsamples'
		while (DLL.check_context() == 1) System.Threading.Thread.Sleep (1);
		status = DLL.RTcmix_setparams(44100, 2, 1024, 1, 32, objno); // '32' is the nmber of RTcmix busses

		// uncommenting this would send a 1.0 second ‘beep’ upon startup

		String score = "bus_config(\"IINOISE\", \"aux 0-1 out\") " +
			"bus_config(\"GVERB\", \"aux 0-1 in\", \"out 0-1\") " +
			"ampenv = maketable(\"line\", 1000, 0,0, 1,1, 2,0) " +
			"GVERB(0, 0, 9999, 0.9, 178.0, 17.0, 0.71, 0.34, -20.0, -11.0, -9.0, 7.0) " +
			"MAXBANG(0) ";
		SendScore(score);
	}


	void Update () {
	}


	void OnApplicationQuit(){
		while (DLL.check_context() == 1) System.Threading.Thread.Sleep (1);
		DLL.RTcmix_destroy (objno); // reset RTcmix
	}


	// this function is called by unity for every buffer of sound samples computed
	void OnAudioFilterRead(float[] data, int channels){
		unsafe{
			fixed (float* b = data) {
				while (DLL.check_context() == 1) System.Threading.Thread.Sleep (1);
				// null input, synthesis only.  If you were processing sound, the (void *)null
				// would be replaced by (void *)b
				// Also, 1024 instead of 2048 because it is frames, not nsamples
				DLL.RTcmix_runAudio((void *)null, (void *)b, 1024, objno);


				// uncomment this to process MAXBANGs
				if (DLL.unity_checkForBang(objno) == 1) {
					String score = "setup(irand(150, 400), 40, 1) " +
						"IINOISE(0, irand(7, 11), 20000*ampenv, 0.5) " +
						"MAXBANG(irand(5, 10)) ";
					SendScore(score);			
				}


				// uncomment this to process MAXMESSAGEs
				/*
				fixed (float *vptr = vals) {
					// this shows how to access the values from a MAXMESSAGE
					int nvals = DLL.unity_checkForVals(vptr, objno);
					if (nvals > 0) {
						Debug.Log("got some vals, " + nvals + " and vals[1] is " + vals[1]);
					}
				}
				*/


				// uncomment this to print RTcmix messages/errors in the system console
				/*
				fixed (char *pptr = printvals) {
					int pcheck = DLL.unity_checkForPrint(pptr, objno);
					while (pcheck > 0) {
						string s = new string(printvals);
						Debug.Log(s);
						Array.Clear(printvals, 0, printvals.Length);
						pcheck = DLL.unity_checkForPrint(pptr, objno);
					}
				}
				*/
			}
		}
	}


	// this is the score-sending function
	public void SendScore(String score) {
		IntPtr sptr = Marshal.StringToHGlobalAnsi(score);
		unsafe {
			while (DLL.check_context() == 1) System.Threading.Thread.Sleep (1);
			DLL.unity_parse_score ((char*)sptr.ToPointer (), score.Length, objno);
		}
		Marshal.FreeHGlobal(sptr);
	}
}

